/*
 * Author: Konrad Rzeszutek <conradus@mindless.com>
 * Date:   Oct 20th 1999
 * File:   Editor3D/Editor3D.java
 *
 * Notes:
 *
 *   This program is USer Interface for the Editor 3D package graphical system.
 * Its a a nice GUI allowing various 2D manipulations of a dictionary file.
 * Servers mainly to please Eduardo Korthright and to get an A :p
 *
 *   Putting jokes aside, this program is the top object of the pyramid, and
 * uses SmartCanvas to draw the object. Refer to that object for more info
 * regarding the use of matrix operations.
 *
 * To run:
 *
 * %java Editor3D.Editor3D
 *
 * (The CLASSPATH *has* to include the directory in which Editor3D directory
 * is located).
 *
 * Disclaimer:
 *
 *  This program cannot be used for illegal purposes. Neither can you use this
 * program for hurting mouses, cats, rats, or other small animals. Big 
 * animals, such as elephants, will be able to protect themselves, so dont
 * try to hurt them. This program cannot be used for designing mass destruction
 * weapons, for any type of A||B||C weapons (yes, that's an 'or' -- it includes
 * all of these weapons). You cannot implement, copy,  any of these parts of
 * this wonderfull program if you planning to incorporate them into
 * Micro$oft platform products. If you planning to use *any* non-Micro$oft
 * platform products, you are more then welcome to copy the copy, ideas, etc.
 *
 * That's about it.
 */
 

package Editor3D;

import java.awt.*;
import java.awt.event.*;

public  class Editor3D extends Frame implements WindowListener, ActionListener, ItemListener, FocusListener {

    // All these lovely variables now:
        
    // Buttons:
    private Button btnRotate;
    private Button btnScale;
    private Button btnShear;
    private Button btnTranslate;
    private Button btnTST;
    private Button btnTRT;
    private Button btnTShT;
    private Button btnWindow;
    // Menu items:
    private MenuItem btnOpen;
    private MenuItem btnReset;
    private MenuItem btnExit;
    // Checkbox'es:
    private CheckboxMenuItem btnLockScale;
    private CheckboxMenuItem btnDebug;    
    private CheckboxMenuItem btnPerspective;
    private CheckboxMenuItem btnBackface;
    private CheckboxMenuItem btnLight;
    // Textfields for all the data
    private TextField fieldRx;
    private TextField fieldRy;
    private TextField fieldRz;
    private TextField fieldSx;
    private TextField fieldSy;
    private TextField fieldSz;
    private TextField fieldTx;
    private TextField fieldTy;
    private TextField fieldTz;
    private TextField fieldShx;
    private TextField fieldShy;
    private TextField fieldShz;
    // And now for U and V:
    private TextField fieldUMin;
    private TextField fieldUMax;
    private TextField fieldVMin;
    private TextField fieldVMax;
    // Now for VRP
    private TextField fieldVRPx;
    private TextField fieldVRPy;
    private TextField fieldVRPz;
    // VPN
    private TextField fieldVPNx;
    private TextField fieldVPNy;
    private TextField fieldVPNz;
    // VUP:
    private TextField fieldVUPx;
    private TextField fieldVUPy;
    private TextField fieldVUPz;
    // COP:
    private TextField fieldCOPx;
    private TextField fieldCOPy;
    private TextField fieldCOPz;

    // Light
    private TextField fieldLx;
    private TextField fieldLy;
    private TextField fieldLz;

    private Object focusObj;
    
    private SmartCanvas canvas;
    
    // And now the ones used for all our stuff:

    private boolean debug = false;
    private boolean uniform_scale = true;
    private boolean perspective = true;
    private boolean backface = false;
    private boolean light = false;
    
    // Our value holders.
    private double[] rotate;
    private double[] scale;
    private double[] translate;
    private double[] shear;
    private double[] v;
    private double[] u;
    private VectorObject aVRP;
    private VectorObject aVPN;
    private VectorObject aVUP;
    private VectorObject aCOP;
    private VectorObject aLight;

    final static int X_POS = 0;
    final static int Y_POS = 1;
    final static int Z_POS = 2;
    final static int MIN   = 0;
    final static int MAX   = 1;
    final static String TITLE = "Editor3D, Filename: ";
    
    /**
     * Constructor.
     */
    public Editor3D () {
    
	super (TITLE+" NONE");
	// Our holders for user-customized values
	scale = new double[3];
	translate = new double[3];
	shear = new double[3];
	rotate = new double[3];
	u = new double[2];
	v = new double[2];
	
	// Needed data:
	aVRP = new VectorObject();
	aVPN = new VectorObject();
	aVUP = new VectorObject();
	aCOP = new VectorObject();
	aLight = new VectorObject();

	setup();
    
    }
    
    /*
     * Opens the file, or not -- depending on the user's action.
     *
     */
    private void openFile() {
    
	FileDialog dialogLoad = new FileDialog(this,"Open a point-face dictionary",FileDialog.LOAD);
	dialogLoad.setFile("*.dict; *.dic");
	dialogLoad.show();
	if (dialogLoad.getFile() != null) {	    
	    setTitle(TITLE+ (new StringBuffer(String.valueOf(dialogLoad.getDirectory())).append(dialogLoad.getFile()).toString()));
       	    canvas.update(debug,perspective, backface, light);
	    canvas.newObject(new StringBuffer(String.valueOf(dialogLoad.getDirectory())).append(dialogLoad.getFile()).toString());
	    reset();	    
	}
    }
    
    /*
     * Refreshes the arrays for translation, rotation, and shaering from
     * the TextFields where the user is suppose to put these info in.
     */
    private void refreshTextFieldFromTSRSh() {
    
	fieldTx.setText(Double.toString(translate[X_POS])); 
	fieldTy.setText(Double.toString(translate[Y_POS])); 
	fieldTz.setText(Double.toString(translate[Z_POS]));
	
	fieldSx.setText(Double.toString(scale[X_POS]));
	fieldSy.setText(Double.toString(scale[Y_POS]));
	fieldSz.setText(Double.toString(scale[Z_POS]));
	
	fieldRx.setText(Double.toString(rotate[X_POS])); 
	fieldRy.setText(Double.toString(rotate[Y_POS]));  
	fieldRz.setText(Double.toString(rotate[Z_POS])); 
	
	fieldShx.setText(Double.toString(shear[X_POS])); 
	fieldShy.setText(Double.toString(shear[Y_POS]));
	fieldShz.setText(Double.toString(shear[Z_POS]));
	
	fieldUMin.setText(Double.toString(u[MIN]));
	fieldUMax.setText(Double.toString(u[MAX]));
	fieldVMin.setText(Double.toString(v[MIN]));
	fieldVMax.setText(Double.toString(v[MAX]));
	
	fieldVRPx.setText(Double.toString(aVRP.vec[X_POS]));
	fieldVRPy.setText(Double.toString(aVRP.vec[Y_POS]));
	fieldVRPz.setText(Double.toString(aVRP.vec[Z_POS]));
	
	fieldVPNx.setText(Double.toString(aVPN.vec[X_POS]));
	fieldVPNy.setText(Double.toString(aVPN.vec[Y_POS]));
	fieldVPNz.setText(Double.toString(aVPN.vec[Z_POS]));
	
	fieldVUPx.setText(Double.toString(aVUP.vec[X_POS]));
	fieldVUPy.setText(Double.toString(aVUP.vec[Y_POS]));
	fieldVUPz.setText(Double.toString(aVUP.vec[Z_POS]));
	
	fieldCOPx.setText(Double.toString(aCOP.vec[X_POS]));
	fieldCOPy.setText(Double.toString(aCOP.vec[Y_POS]));
	fieldCOPz.setText(Double.toString(aCOP.vec[Z_POS]));

	fieldLx.setText(Double.toString(aLight.vec[X_POS])); 
	fieldLy.setText(Double.toString(aLight.vec[Y_POS])); 
	fieldLz.setText(Double.toString(aLight.vec[Z_POS]));

    }

    /*
     * Refreshes the TextFields for translation, rotation, and shaering 
     * VPN, VRP, ... , from the appropiate arrays.
     */
    
    private void refreshTSRShFromTextField () {    

	try {

	    scale[X_POS] = Double.valueOf(fieldSx.getText()).doubleValue();
	    scale[Y_POS] = Double.valueOf(fieldSy.getText()).doubleValue();
	    scale[Z_POS] = Double.valueOf(fieldSz.getText()).doubleValue();

    	    shear[X_POS] = Double.valueOf(fieldShx.getText()).doubleValue();
	    shear[Y_POS] = Double.valueOf(fieldShy.getText()).doubleValue();
	    shear[Z_POS] = Double.valueOf(fieldShz.getText()).doubleValue();
	    
    	    rotate[X_POS] = Double.valueOf(fieldRx.getText()).doubleValue();
	    rotate[Y_POS] = Double.valueOf(fieldRy.getText()).doubleValue();
	    rotate[Z_POS] = Double.valueOf(fieldRz.getText()).doubleValue();
	    
    	    translate[X_POS] = Double.valueOf(fieldTx.getText()).doubleValue();
	    translate[Y_POS] = Double.valueOf(fieldTy.getText()).doubleValue();
	    translate[Z_POS] = Double.valueOf(fieldTz.getText()).doubleValue();
	
	        
	    u[MIN] = Double.valueOf(fieldUMin.getText()).doubleValue();
	    u[MAX] = Double.valueOf(fieldUMax.getText()).doubleValue();	    
	    
	    v[MIN] = Double.valueOf(fieldVMin.getText()).doubleValue();
	    v[MAX] = Double.valueOf(fieldVMax.getText()).doubleValue();

       	    aVRP.vec[X_POS] = Double.valueOf(fieldVRPx.getText()).doubleValue();
	    aVRP.vec[Y_POS] = Double.valueOf(fieldVRPy.getText()).doubleValue();
	    aVRP.vec[Z_POS] = Double.valueOf(fieldVRPz.getText()).doubleValue();
	    
	    aVPN.vec[X_POS] = Double.valueOf(fieldVPNx.getText()).doubleValue();
	    aVPN.vec[Y_POS] = Double.valueOf(fieldVPNy.getText()).doubleValue();
	    aVPN.vec[Z_POS] = Double.valueOf(fieldVPNz.getText()).doubleValue();
	    
	    aVUP.vec[X_POS] = Double.valueOf(fieldVUPx.getText()).doubleValue();
	    aVUP.vec[Y_POS] = Double.valueOf(fieldVUPy.getText()).doubleValue();
	    aVUP.vec[Z_POS] = Double.valueOf(fieldVUPz.getText()).doubleValue();

       	    aCOP.vec[X_POS] = Double.valueOf(fieldCOPx.getText()).doubleValue();
	    aCOP.vec[Y_POS] = Double.valueOf(fieldCOPy.getText()).doubleValue();
	    aCOP.vec[Z_POS] = Double.valueOf(fieldCOPz.getText()).doubleValue();
	    aLight.vec[X_POS] = Double.valueOf(fieldLx.getText()).doubleValue();

	    aLight.vec[Y_POS] = Double.valueOf(fieldLy.getText()).doubleValue();
	    aLight.vec[Z_POS] = Double.valueOf(fieldLz.getText()).doubleValue();


	} catch (NumberFormatException e) {
	    // System.out.println("What kind of $#@$#@@ data is this?");	    
	}		    
    }
    
    /*
     * Kills the Editor3D program.
     */
    private void die() {
    
	setVisible(false);
	dispose();
	System.exit(0);
    }
    
    /*
     * Discards all the modifications done by the user. Reverts
     * all the values to the original form.
     */
    private void reset() {
    

	scale[X_POS]     = scale [Y_POS]    = scale[Z_POS]     = 1.0;
	rotate[X_POS]    = rotate[Y_POS]    = rotate[Z_POS]    = 0.0;
	translate[X_POS] = translate[Y_POS] = translate[Z_POS] = 0.0;
	shear[X_POS]     = shear[Y_POS]     = shear[Z_POS]     = 0.0;
	
	canvas.reset();
	
	u = canvas.getU();
	v = canvas.getV();

	aVRP = canvas.getVRP();
	aVPN = canvas.getVPN();
	aVUP = canvas.getVUP();
	aCOP = canvas.getCOP();
	aLight = canvas.getLight();

	refreshTextFieldFromTSRSh();

	canvas.update(u,v,aVRP,aVPN,aVUP,aCOP,aLight,debug,perspective, backface, light);	    

    }
    /*
     * Checks to see if the shear fields have either X or Y or Z filled or
     * XY or XZ or YZ fields filled.
     * 
     * return:
     *  true || false
     */
    private boolean canShear() {
    
	if ( (shear[X_POS] == shear[Y_POS]) && (shear[Y_POS] == shear[Z_POS]) && (shear[Z_POS] != 0.00)) {
	    // Show the dialog window
	    DialogBox box = new DialogBox(this,"Don't know which way to shear! You must leave one of the fields empty.");
	    return false;
	} 
	return true;
    }
    
    /*
     * Perform an action when the user clicks on the buttons, menus
     * or textfields.
     */    
    public void actionPerformed(ActionEvent actionEvent) {
    
	Object event = actionEvent.getSource();
	
	if (event instanceof Button) { // Lots of them	
	    
	    refreshTSRShFromTextField(); // Meeded to get the right UV.

	    canvas.update(debug,perspective, backface, light);
	    
	    if (event==btnRotate) canvas.rotateObject(rotate);
	    if (event==btnScale)  canvas.scaleObject(scale);
	    if (event==btnShear)  if (canShear()) canvas.shearObject(shear);
	    if (event==btnTranslate) canvas.translateObject(translate);
	    if (event==btnTRT)    canvas.trtObject(translate,rotate);
	    if (event==btnTST)    canvas.tstObject(translate,scale); 
	    if (event==btnTShT)   if (canShear()) canvas.tshtObjet(translate,shear);	    
	    if (event==btnWindow) canvas.update(u,v,aVRP,aVPN,aVUP,aCOP,aLight,debug,perspective, backface, light);	    
	}
	
	if (event instanceof MenuItem) { // not that too many of these

	    if (event==btnOpen) openFile();
	    if (event==btnReset) reset();
	    if (event==btnExit) die();
	}
	
	if (event instanceof TextField) {    
	 
	    if ( (event == fieldSx) || (event == fieldSx) || (event == fieldSz)) {

		if (event == fieldSx) scale[Z_POS] = scale[Y_POS] = scale[X_POS] = Double.valueOf(fieldSx.getText()).doubleValue();
		if (event == fieldSy) scale[Z_POS] = scale[Y_POS] = scale[X_POS] = Double.valueOf(fieldSy.getText()).doubleValue();
		if (event == fieldSz) scale[Z_POS] = scale[Y_POS] = scale[X_POS] = Double.valueOf(fieldSz.getText()).doubleValue();
		// Put all new info on the screen:
		refreshTextFieldFromTSRSh();
	    }
	    // Update the values.
	    refreshTSRShFromTextField();	    	    
	}
    }

    /*
     * Perform the some function when the user clicks on these
     * lock-able menuitem's in the menubar.
     */
    public void itemStateChanged(ItemEvent itemEvent) {
	
	Object e = itemEvent.getSource();

	if (e == btnDebug) 
	    if (btnDebug.getState()) debug = true; else debug = false;
	
	if (e == btnLockScale) 
	    if (btnLockScale.getState()) uniform_scale= true; else uniform_scale = false;	
	
	if ((e != btnDebug) || (e != btnLockScale)) {
	    if (e == btnPerspective)
		if (btnPerspective.getState()) perspective= true; else perspective = false;	
	    if (e == btnBackface) 
		if (btnBackface.getState()) backface = true; else backface = false;
	    if (e == btnLight)
		if (btnLight.getState()) light = true; else light = false;
	    // Update the window.		    
	    canvas.update(u,v,aVRP,aVPN,aVUP,aCOP,aLight,debug,perspective, backface, light);
	}
    }
    
    /*
     * When the users focuses his/her attention on the object, invoke
     * this method.
     */
    public void focusGained(FocusEvent event) {

	// Save the object (to be used in focusLost).
	focusObj = event.getSource();
    }
    
    /*
     * Perform some operation when the user loses his/her attention to
     * the object.
     */     
    public void focusLost(FocusEvent event) {

	if ((uniform_scale) && ( (focusObj == fieldSx) || (focusObj == fieldSy) || (focusObj == fieldSz))) {

	    if (focusObj == fieldSx) scale[Z_POS] = scale[Y_POS] = scale[X_POS] = Double.valueOf(fieldSx.getText()).doubleValue();
    	    if (focusObj == fieldSy) scale[Z_POS] = scale[Y_POS] = scale[X_POS] = Double.valueOf(fieldSy.getText()).doubleValue();
	    // For future:
	    if (focusObj == fieldSz) scale[Z_POS] = scale[Y_POS] = scale[X_POS] = Double.valueOf(fieldSz.getText()).doubleValue();
	    // Make it appear!
	    refreshTextFieldFromTSRSh();
	}
	// Update the rest.
	 refreshTSRShFromTextField ();

    }
    
    /* 
     * Make the GUI.
     */
    public void setup () {
    
       
        setSize(700, 550);
        setBackground(Color.white);
        setLayout(new BorderLayout());	
	
        btnRotate = new Button("Rotation");
        btnRotate.addActionListener(this);
	// Check this out.
	
	btnScale = new Button("Scale");
        btnScale.addActionListener(this);
	
	btnTranslate= new Button("Translate");
        btnTranslate.addActionListener(this);
		
        btnShear = new Button("Shear");
        btnShear.addActionListener(this);

	btnWindow = new Button("Apply");
        btnWindow.addActionListener(this);
	
	// The composite buttons:
        btnTRT = new Button("T/R/T");
        btnTRT.addActionListener(this);

	btnTST = new Button("T/S/T");
        btnTST.addActionListener(this);

        btnTShT = new Button("T/Sh/T");
        btnTShT.addActionListener(this);
	
	// Rotate Fields:
	
	fieldRx = new TextField("0.0");
        fieldRx.addActionListener(this);
	fieldRx.addFocusListener(this);
	
	fieldRy = new TextField("0.0");
	fieldRy.addActionListener(this);
	fieldRy.addFocusListener(this);
	
        fieldRz = new TextField("0.0");
	fieldRz.addActionListener(this);
	fieldRz.addFocusListener(this);
	
	// Shear fields:
        fieldSx = new TextField("1.0");
        fieldSx.addActionListener(this);
	fieldSx.addFocusListener(this);
	
        fieldSy = new TextField("1.0");
        fieldSy.addActionListener(this);
	fieldSy.addFocusListener(this);
	
        fieldSz = new TextField("1.0");
        fieldSz.addActionListener(this);
	fieldSz.addFocusListener(this);
		
	// Translate fields:
        fieldTx = new TextField("0.0");
	fieldTx.addActionListener(this);
	fieldTx.addFocusListener(this);
	
        fieldTy = new TextField("0.0");
	fieldTy.addActionListener(this);
	fieldTy.addFocusListener(this);
	
	fieldTz = new TextField("0.0");	
	fieldTz.addActionListener(this);
	fieldTz.addFocusListener(this);
	
	// Shear fields:
        fieldShx = new TextField("0.0");
	fieldShx.addActionListener(this);
	fieldShx.addFocusListener(this);
	
        fieldShy = new TextField("0.0");    
	fieldShy.addActionListener(this);
	fieldShy.addFocusListener(this);
	
        fieldShz = new TextField("0.0");
	fieldShz.addActionListener(this);
	fieldShz.addFocusListener(this);
	

	fieldLx = new TextField("");
	fieldLx.addActionListener(this);
	fieldLx.addFocusListener(this);


	fieldLy = new TextField("");
	fieldLy.addActionListener(this);
	fieldLy.addFocusListener(this);


	fieldLz = new TextField("");
	fieldLz.addActionListener(this);
	fieldLz.addFocusListener(this);

	// The place where we will draw
	canvas = new SmartCanvas();
        canvas.setSize(SmartCanvas.X_MAX,SmartCanvas.Y_MAX);
        canvas.setBackground(Color.black);
        canvas.setForeground(Color.white);
	
	// Panels for putting all this stuff up.
	Panel _panelCenter = new Panel();
        _panelCenter.setLayout(new FlowLayout());
        _panelCenter.add(canvas);
	
	// The bottom buttons and such:
        Panel _panelSouth = new Panel();
        _panelSouth.setLayout(new GridLayout(4, 5));
	
        _panelSouth.add(new Label("Rx / Ry / Rz"));
        _panelSouth.add(fieldRx);  
        _panelSouth.add(fieldRy);
        _panelSouth.add(fieldRz);
        _panelSouth.add(btnRotate);
        _panelSouth.add(btnTRT);
        _panelSouth.add(new Label("Sx / Sy / Sz"));
        _panelSouth.add(fieldSx);
        _panelSouth.add(fieldSy);
        _panelSouth.add(fieldSz);
        _panelSouth.add(btnScale);
        _panelSouth.add(btnTST);
        _panelSouth.add(new Label("Shx / Shy / Shz"));
        _panelSouth.add(fieldShx);
        _panelSouth.add(fieldShy);
        _panelSouth.add(fieldShz);
        _panelSouth.add(btnShear);
        _panelSouth.add(btnTShT);
        _panelSouth.add(new Label("Tx / Ty / Tz"));
        _panelSouth.add(fieldTx);
        _panelSouth.add(fieldTy);
        _panelSouth.add(fieldTz);
        _panelSouth.add(btnTranslate);
        //_panelSouth.add(new Label(" "));

		// Window fields:
	
        fieldUMin = new TextField();
        fieldUMax = new TextField();
        fieldVMin = new TextField();
        fieldVMax = new TextField();
	fieldVRPx = new TextField();	
	fieldVRPy = new TextField();
	fieldVRPz = new TextField();
	fieldVPNx = new TextField();	
	fieldVPNy = new TextField();
	fieldVPNz = new TextField();
	
	fieldVUPx = new TextField();	
	fieldVUPy = new TextField();
	fieldVUPz = new TextField();

	fieldCOPx = new TextField();	
	fieldCOPy = new TextField();
	fieldCOPz = new TextField();
	
	
        Panel _panelEast = new Panel();
        _panelEast.setLayout(new GridLayout(13, 4));
	_panelEast.add(new Label("VRPx  "));
        _panelEast.add(fieldVRPx);
	
	_panelEast.add(new Label("COPx  "));
        _panelEast.add(fieldCOPx);
	
	_panelEast.add(new Label("VRPy  "));
        _panelEast.add(fieldVRPy);
       	
	_panelEast.add(new Label("COPy  "));
        _panelEast.add(fieldCOPy);

	_panelEast.add(new Label("VRPz  "));
        _panelEast.add(fieldVRPz);


	_panelEast.add(new Label("COPz  "));
        _panelEast.add(fieldCOPz);


	_panelEast.add(new Label(" "));
	_panelEast.add(new Label(" "));
	_panelEast.add(new Label(" "));
	_panelEast.add(new Label(" "));


	_panelEast.add(new Label("VPNx  "));
        _panelEast.add(fieldVPNx);

	_panelEast.add(new Label("Lx  "));
        _panelEast.add(fieldLx);


	_panelEast.add(new Label("VPNy  "));
        _panelEast.add(fieldVPNy);

	_panelEast.add(new Label("Ly  "));
        _panelEast.add(fieldLy);

	_panelEast.add(new Label("VPNz  "));
        _panelEast.add(fieldVPNz);
	
	_panelEast.add(new Label("Lz  "));
        _panelEast.add(fieldLz);


	_panelEast.add(new Label(" "));
	_panelEast.add(new Label(" "));
	_panelEast.add(new Label(" "));
	_panelEast.add(new Label(" "));

	_panelEast.add(new Label("VUPx  "));
        _panelEast.add(fieldVUPx);

	_panelEast.add(new Label("Umin  "));
        _panelEast.add(fieldUMin);

	_panelEast.add(new Label("VUPy  "));
        _panelEast.add(fieldVUPy);

	_panelEast.add(new Label("Umax  "));
        _panelEast.add(fieldUMax);

	_panelEast.add(new Label("VUPz  "));
        _panelEast.add(fieldVUPz);	

     
        _panelEast.add(new Label("Vmin  "));
        _panelEast.add(fieldVMin);
	_panelEast.add(new Label(" "));
	_panelEast.add(new Label(" "));

        _panelEast.add(new Label("Vmax  "));	
        _panelEast.add(fieldVMax);
	_panelEast.add(new Label(" "));
	_panelEast.add(new Label(" "));
	_panelEast.add(new Label(" "));

	// Just to make blank space.
        _panelEast.add(btnWindow);
	
        add(_panelSouth,BorderLayout.SOUTH);
        add(_panelEast,BorderLayout.EAST);
        add(_panelCenter,BorderLayout.WEST);
	
	// Menubar
        btnLockScale = new CheckboxMenuItem("Uniform Scale", true);
        btnLockScale.addItemListener(this);
	
	btnPerspective = new CheckboxMenuItem("Perspective view", true);
	btnPerspective.addItemListener(this);
	
        btnDebug = new CheckboxMenuItem("Show Transform Matrix", false);
        btnDebug.addItemListener(this);
	
	btnBackface = new CheckboxMenuItem("Draw Backfaces", false);
	btnBackface.addItemListener(this);
	
	btnLight = new CheckboxMenuItem("Lighting", false);
	btnLight.addItemListener(this);
	
        MenuBar menubar = new MenuBar();
        Menu menuObj = new Menu("Object");
	// Open the file:
        btnOpen = new MenuItem("Open Object");
        btnOpen.addActionListener(this);
	// Reset the entries:
        btnReset = new MenuItem("Reset Object");
        btnReset.addActionListener(this);
	// and Exit.
        btnExit = new MenuItem("Exit");
        btnExit.addActionListener(this);
	
        menuObj.add(btnOpen);
	menuObj.add(btnReset);
	menuObj.add(btnLockScale);
	menuObj.add(btnPerspective);
	menuObj.add(btnDebug);
	menuObj.add(btnBackface);
	menuObj.add(btnLight);
	menuObj.add(btnExit);
        menubar.add(menuObj);
        setMenuBar(menubar);

        setResizable(false);
	//reset();
        setVisible(true);
        addWindowListener(this);
    
    }

    // These are needed.
    public void windowActivated(WindowEvent windowEvent)
    {
    }

    public void windowClosed(WindowEvent windowEvent)
    {
    }

    public void windowClosing(WindowEvent windowEvent)
    {
        die();
    }

    public void windowDeactivated(WindowEvent windowEvent)
    {
    }

    public void windowDeiconified(WindowEvent windowEvent)
    {
    }

    public void windowIconified(WindowEvent windowEvent)
    {
    }

    public void windowOpened(WindowEvent windowEvent)
    {
    }

    public static void main(String args[]) {
    
	Editor3D editor = new Editor3D();
}

}
